program test

#include "petsc/finclude/petscvec.h"
#include "petsc/finclude/petscdmplex.h"
#include "petsc/finclude/petscdmlabel.h"
 use petscvec
 use petscdmplex
 use petscsys

 implicit none

 DM             :: dm_dumm, dm_geom, dm_geom_cell
 Vec            :: vglo, vloc
 Mat            :: Map
 PetscMPIInt    :: myid, nproc
 PetscErrorCode :: ierr
 PetscSection   :: section
 PetscInt       :: dim
 PetscFE        :: fe_0, fe_1
 character(100) :: fname
 integer        :: i
 integer        :: pst, pend
 integer        :: vst, vend
 integer        :: cst, cend


 call PetscInitialize(PETSC_NULL_CHARACTER, ierr)
 if(ierr.ne.0) then
   write(*,*) 'Fail to initialize PETSc ...'
   stop
 endif
 call MPI_COMM_RANK(MPI_COMM_WORLD, myid, ierr)
 call MPI_COMM_SIZE(MPI_COMM_WORLD, nproc, ierr)
 write(*,*) 'MPI information ... ', myid, nproc


 fname = './square.msh'
 call DMPlexCreateGmshFromFile(PETSC_COMM_WORLD, fname, PETSC_TRUE, dm_dumm, ierr);CHKERRA(ierr)
 call DMPlexDistributeSetDefault(dm_dumm, PETSC_FALSE, ierr);CHKERRA(ierr)
 call DMPlexDistribute(dm_dumm, 0, PETSC_NULL_SF, dm_geom, ierr);CHKERRA(ierr)
 call DMGetDimension(dm_geom, dim, ierr);CHKERRA(ierr)
 call DMClone(dm_geom, dm_geom_cell, ierr);CHKERRA(ierr)
 ! dm_geom      : solution on vertex
 ! dm_geom_cell : solution on cell
 ! mapping between them needed


 ! setup sections
 ! (1) define section with 1-dof on cell for cell-dm
 call PetscSectionCreate(PETSC_COMM_WORLD, section, ierr);CHKERRA(ierr)
 call PetscSectionSetNumFields(section, 1, ierr);CHKERRA(ierr)
 call PetscSectionSetFieldComponents(section, 0, 1, ierr);CHKERRA(ierr)
 call DMPlexGetDepthStratum(dm_geom_cell, 2, cst, cend, ierr);CHKERRA(ierr) ! cells
 call PetscSectionSetChart(section, cst, cend, ierr);CHKERRA(ierr)
 do i=cst,cend-1
   call PetscSectionSetDof(section, i, 1, ierr);CHKERRA(ierr)
   call PetscSectionSetFieldDof(section, i, 0, 1, ierr);CHKERRA(ierr)
 enddo
 call PetscSectionSetUp(section, ierr);CHKERRA(ierr)
 call DMSetLocalSection(dm_geom_cell, section, ierr);CHKERRA(ierr)
 call PetscSectionDestroy(section, ierr);CHKERRA(ierr)


 ! (2) define section with 1-dof on vertex for vertex-dm
 call PetscSectionCreate(PETSC_COMM_WORLD, section, ierr);CHKERRA(ierr)
 call PetscSectionSetNumFields(section, 1, ierr);CHKERRA(ierr)
 call PetscSectionSetFieldComponents(section, 0, 1, ierr);CHKERRA(ierr)
 call DMPlexGetDepthStratum(dm_geom, 0, vst, vend, ierr);CHKERRA(ierr) ! vertex
 call PetscSectionSetChart(section, vst, vend, ierr);CHKERRA(ierr)
 do i=vst,vend-1
   call PetscSectionSetDof(section, i, 1, ierr);CHKERRA(ierr)
   call PetscSectionSetFieldDof(section, i, 0, 1, ierr);CHKERRA(ierr)
 enddo
 call PetscSectionSetUp(section, ierr);CHKERRA(ierr)
 call DMSetLocalSection(dm_geom, section, ierr);CHKERRA(ierr)
 call PetscSectionDestroy(section, ierr);CHKERRA(ierr)


 ! FE object for P1 for dm_geom (i.e., vertex dm)
 call PetscFECreateLagrange(PETSC_COMM_SELF, dim, 1, PETSC_FALSE, 1, PETSC_DETERMINE, fe_1, ierr);CHKERRA(ierr)
 call DMSetField(dm_geom, 0, PETSC_NULL_DMLABEL, fe_1, ierr);CHKERRA(ierr)
 call DMCreateDS(dm_geom, ierr);CHKERRA(ierr)
 call PetscFEDestroy(fe_1, ierr);CHKERRA(ierr)

 ! FE for P0 for dm_geom_cell (i.e., cell dm)
 call PetscFECreateLagrange(PETSC_COMM_SELF, dim, 1, PETSC_FALSE, 0, PETSC_DETERMINE, fe_0, ierr);CHKERRA(ierr)
 call DMSetField(dm_geom_cell, 0, PETSC_NULL_DMLABEL, fe_0, ierr);CHKERRA(ierr)
 call DMCreateDS(dm_geom_cell, ierr);CHKERRA(ierr)
 call PetscFEDestroy(fe_0, ierr);CHKERRA(ierr)


 ! create interpolation --> Failed
 call DMCreateInterpolation(dm_geom_cell, dm_geom, Map, ierr);CHKERRA(ierr)




 call PetscFinalize(ierr)
 stop
end program test
